import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jfd.exp.*;

public class example2 extends JFrame 
   implements ActionListener, DrawErrorObserver, AngleChangeObserver
{
   private JTextField expression      = null;
   private AngleControl xAngleControl = null;
   private AngleControl yAngleControl = null;
   private AngleControl zAngleControl = null;
   private DrawPanel drawPanel        = null;

   private static final int X_ANGLE_DEFAULT  = 140;
   private static final int Y_ANGLE_DEFAULT  = 160;
   private static final int Z_ANGLE_DEFAULT  = 70;
   private static final int X_DIVISIONS      = 20;
   private static final int Y_DIVISIONS      = 20;
   private static final double VALUE         = 100.0;
   private static final double X_START       = -VALUE;
   private static final double Y_START       = -VALUE;
   private static final double X_END         = VALUE;
   private static final double Y_END         = VALUE;

   public example2()
   {
      addWindowListener(new AppCloser());

      setSize(640, 480);

      JPanel contentPane = (JPanel)getContentPane();

      contentPane.setLayout(new BorderLayout());
      contentPane.add("South", makeBottomPanel());
      contentPane.add("Center", drawPanel = new DrawPanel());

      pack();
      expression.requestFocusInWindow();
      setVisible(true);
   }

   private JPanel makeBottomPanel()
   {
      // Create a panel containing the expression entry text field
      // and its label

      JPanel p1  = new JPanel(new FlowLayout());
      expression = new JTextField(40);

      expression.addActionListener(this);

      p1.add(new JLabel("Enter a function of x and y: "));
      p1.add(expression);

      p1.setBorder(BorderFactory.createEtchedBorder());

      // Create a panel displaying the angle controls

      JPanel p2 = new JPanel(new GridLayout(3, 1));

      xAngleControl = new AngleControl(X_ANGLE_DEFAULT, "xAxis", this);
      yAngleControl = new AngleControl(Y_ANGLE_DEFAULT, "yAxis", this);
      zAngleControl = new AngleControl(Z_ANGLE_DEFAULT, "zAxis", this);

      p2.add(xAngleControl);
      p2.add(yAngleControl);
      p2.add(zAngleControl);

      p2.setBorder(
         BorderFactory.createTitledBorder(
            BorderFactory.createEtchedBorder(),
            "Set Viewing Angles"
         )
      );

      // Combine the two panels

      JPanel p3 = new JPanel(new BorderLayout());

      p3.add("South", p1);
      p3.add("North", p2);

      return(p3);
   }

   private void draw()
   {
      String s = expression.getText().trim();

      if(s.equals(""))
         return;

      Node n = null;

      try
      {
         n = ExpressionParser.parse(s);
      }
      catch(Throwable t)
      {
         JOptionPane.showMessageDialog(
            this, 
            t.getMessage(),
            "Parse Error",
            JOptionPane.WARNING_MESSAGE
         );

         return;
      }

      drawPanel.draw(
         xAngleControl.getAngle(),
         yAngleControl.getAngle(),
         zAngleControl.getAngle(),
         X_DIVISIONS,
         Y_DIVISIONS,
         X_START,
         Y_START,
         X_END,
         Y_END,
         n,
         this
      );
   }

   public void angleChanged(int newAngle)
   {
      draw();
   }

   public void actionPerformed(ActionEvent e)
   {
      draw();
   }

   public void drawErrorOccurred(String s)
   {
      JOptionPane.showMessageDialog(
         this, 
         s,
         "Draw Error",
         JOptionPane.WARNING_MESSAGE
      );
   }

   public static void main(String args[])
   {
      new example2();
   }
}
