package jfd.exp;

import java.util.Hashtable;
import java.util.Vector;
import java.lang.reflect.Method;

public class MethodNode extends Node
{
   private String name                      = null;
   private Node arguments[]                 = null;
   private Method m                         = null;
   private static Hashtable methodTable     = null;
   private static String availableMethods[] = null;

   static
   {
      methodTable = new Hashtable();

      Method methods[] = Math.class.getDeclaredMethods();
      Vector v         = new Vector();

      for(int i = 0; i < methods.length; i ++)
      {
         if(methods[i].getReturnType() == double.class)
         {
            Class types[] = methods[i].getParameterTypes();

            int j = 0;

            while(j < types.length)
            {
               if(types[j] != double.class)
                  break;

               j++;
            }

            if(j > 0 && j == types.length)
            {
               String name = methods[i].getName();

               methodTable.put(name + "|" + j, methods[i]);
               v.addElement(name + "(" + getTypes(j) + ")");
            }
         }
      }

      int size         = v.size();
      availableMethods = new String[size];

      for(int i = 0; i < size; i ++)
         availableMethods[i] = (String)v.elementAt(i);
   }

   private static String getTypes(int nDoubles)
   {
      String s = "";

      for(int i = 0; i < nDoubles; i ++)
         s += ((i == 0 ? "" : ", ") + "double");

      return(s);
   }

   public MethodNode(String name, Node arguments[])
   {
      this.name      = name;
      this.arguments = arguments;
   }

   public void accept(Visitor v)
   {
      v.visited(this);
   }

   public String toString()
   {
      StringBuffer sb = new StringBuffer("");

      sb.append(name + "(");

      for(int i = 0; i < arguments.length; i ++)
      {
         if(i > 0)
            sb.append(", ");

         sb.append(arguments[i]);
      }

      sb.append(")");


      return(sb.toString());
   }

   public static String[] getAvailableMethods()
   {
      return(availableMethods);
   }

   private Method getMethod() throws Exception
   {
      if(m == null)
      {
         m = (Method)methodTable.get(name + "|" + arguments.length);

         if(m == null)
         {
            throw new Exception(
               "No such method: " +
               name + "(" + getTypes(arguments.length) + ")"
            );
         }
      }

      return(m);
   }

   public double execute(
      SymbolTable table
   )
      throws Exception
   {
      double returnValue = 0.0;

      Object values[] = new Object[arguments.length];

      for(int i = 0; i < values.length; i ++)
         values[i] = new Double(arguments[i].execute(table));

      returnValue =
         ((Double)(getMethod().invoke(null, values))).doubleValue();

      return(returnValue);
   }
}
